//*************************************************************************
//* ===============
//*  CTDlayer Class
//* ===============
//*
//* (Description)
//*    Charged track detector layer class.
//* (Requires)
//* (Provides)
//* 	class CTDlayer
//* (Update Recored)
//*    1999/11/08  K.Fujii	Original version.
//*
//*************************************************************************
//
#include "CTDLayer.h"

   const Double_t CTDLayer::kX0Be  = 0.028;	// X0_Be
   const Double_t CTDLayer::kX0Air = 0.000035;	// X0_air
   const Double_t CTDLayer::kX0Si  = 0.107;	// X0_Si
   const Double_t CTDLayer::kX0CFRP= 0.030;	// X0_CFRP
   const Double_t CTDLayer::kX0CDC = 0.000055;	// X0_CDCgas
   const Double_t CTDLayer::kCms   = 0.0141;	// MS constant
   const Double_t CTDLayer::kCptor = 333.56;	// P-to-R constant

//_____________________________________________________________________
//  --------
//  CTDLayer
//  --------
//

ClassImp(CTDLayer)

//_____________________________________________________________________
CTDLayer::CTDLayer(Char_t *s, Int_t n, Double_t r, Double_t t,
		   Double_t b, Double_t x0, Double_t sg) :
   	fName(s), fNsample(n), fThickness(t), fSigmaX(sg),
   	fX0(x0), fB(b), fRin(r), fRout(r+t) {}

CTDLayer::CTDLayer(CTDLayer &l, Double_t r) : 
   	fName(l.fName), fNsample(l.fNsample), fThickness(l.fThickness),
   	fSigmaX(l.fSigmaX), fX0(l.fX0), fB(l.fB), 
   	fRin(r), fRout(r+l.fThickness) {}

CTDLayer::CTDLayer(const CTDLayer &l) :
   	fName(l.fName), fNsample(l.fNsample), fThickness(l.fThickness),
   	fSigmaX(l.fSigmaX), fX0(l.fX0), fB(l.fB),
   	fRin(l.fRin), fRout(l.fRout) {}

//_____________________________________________________________________
void CTDLayer::DebugPrint(const Char_t *opt)
{
   cerr.setf(ios::left);
   cerr.setf(ios::fixed);
   cerr << setw(7) << fName;
   cerr.unsetf(ios::left);
   cerr.setf(ios::right);
   cerr << setprecision(2)
        << " Rin: "  << setw(6) << fRin
        << " Rout: " << setw(6) << fRout
        << " Nsample: " << setw(2) << fNsample << setprecision(4)
        << " sigma_x: " << setw(7) << fSigmaX  << setprecision(5)
        << " X/X0: "    << setw(8) << fX0*fThickness << endl;
   cerr.unsetf(ios::right);
   cerr.unsetf(ios::fixed);
}

//_____________________________________________________________________
TMatrix CTDLayer::GetEfull(Double_t p, Double_t tanl)
{
   TMatrix Error(3,3);
   if (fNsample < 3) {
      cerr << "CTDLayer::GetEfull ...." << endl;
      cerr << "    Nsamle = " << fNsample << " is too small" << endl;
      return Error;
   }
   Double_t n     = fNsample - 1;
   Double_t np1   = fNsample;
   Double_t np2   = n + 2;
   Double_t np3   = n + 3;
   Double_t nm1   = n - 1;
   Double_t tnp1  = 2*n + 1;
   Double_t np123 = np1*np2*np3;  
   Double_t nm123 = nm1*np123;
   Double_t n3    = n*n*n;
   Double_t ptor  = kCptor/fB;
   Double_t ptor2 = ptor*ptor;
   Double_t c     = kCms;
   Double_t pt    = p/TMath::Sqrt(1+tanl*tanl);
   Double_t kappa = 1/pt;
   Double_t l     = fThickness;
   Double_t l2    = l*l;
   Double_t l3    = l2*l;
   Double_t l4    = l3*l;
   Double_t k     = TMath::Power(c*kappa,2)*fX0*TMath::Sqrt(1+tanl*tanl);
   Double_t f     = k*TMath::Power(l,3)*TMath::Power(n,2);
   Double_t sgx2  = fSigmaX*fSigmaX;
   
   TMatrix EB(3,3);			// E_meas/sigma_x^2
   EB(0,0) = 3*(3*n*np1+2)/np123;
   EB(0,1) = 18*n*tnp1/l/np123;
   EB(0,2) = 60*ptor*n*n/l2/np123;
   EB(1,1) = 12*n*tnp1*(8*n-3)/l2/nm123;
   EB(1,2) = 360*ptor*n3/l3/nm123;
   EB(2,2) = 720*ptor2*n3/l4/nm123;
   EB(1,0) = EB(0,1);
   EB(2,0) = EB(0,2);
   EB(2,1) = EB(1,2);
  
   TMatrix ABA(3,3);
   ABA(0,0) =  1/20.;
   ABA(0,1) = -13*l/360;
   ABA(0,2) =  71*l2/5040/ptor;
   ABA(1,1) =  11*l2/420;
   ABA(1,2) = -59*l3/5760/ptor;
   ABA(2,2) =  13*l4/3240/ptor2;
   ABA(1,0) =  ABA(0,1);
   ABA(2,0) =  ABA(0,2);
   ABA(2,1) =  ABA(1,2);

   TMatrix Ems(EB);			// E_MS
   ((Ems *= f) *= ABA) *= EB;
   ((Error  = EB) *= sgx2) += Ems;	// E_meas + E_MS

   return Error;
}

//_____________________________________________________________________
TMatrix CTDLayer::GetDelEms(Double_t p, Double_t tanl)
{
   TMatrix DelEms(3,3);	// Contribution to E_MS from this layer
   if (fNsample > 2) {
      cerr << "CTDLayer::GetEfull ...." << endl;
      cerr << "    Nsamle = " << fNsample << " is too large" << endl;
      return DelEms;
   }
   Double_t ptor  = kCptor/fB;
   Double_t ptor2 = ptor*ptor;
   Double_t c     = kCms;
   Double_t pt    = p/TMath::Sqrt(1.+tanl*tanl);
   Double_t kappa = 1/pt;
   Double_t k1    = TMath::Power(c*kappa,2)*fX0*TMath::Sqrt(1+tanl*tanl);
   Double_t k2    = k1*kappa*kappa*tanl*tanl/(1+tanl*tanl);
   Double_t dy    = fThickness;

   DelEms(0,0) =   k1*TMath::Power(dy,3)/3 + k2*TMath::Power(dy,5)/ptor2/20;
   DelEms(1,0) = - k1*TMath::Power(dy,2)/2 - k2*TMath::Power(dy,4)/ptor2/8;
   DelEms(2,0) =                           + k2*TMath::Power(dy,3)/ptor/6;
   DelEms(1,1) =   k1*dy                   + k2*TMath::Power(dy,3)/ptor2/3;
   DelEms(2,1) =                           - k2*TMath::Power(dy,2)/ptor/2;
   DelEms(2,2) =                             k2*dy;
   DelEms(0,1) = DelEms(1,0);
   DelEms(0,2) = DelEms(2,0);
   DelEms(1,2) = DelEms(2,1);

   return DelEms;
}
